package info.textgrid.lab.collatex.ui.parts;

import info.textgrid.lab.collatex.CollateXPlugin;
import info.textgrid.lab.collatex.model.CollationResults;

import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.StyledString.Styler;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.TextStyle;

import com.google.common.base.Function;
import com.google.common.base.Joiner;
import com.google.common.collect.Iterables;

import eu.interedition.collatex.Token;
import eu.interedition.collatex.Witness;
import eu.interedition.collatex.simple.SimpleToken;

/**
 * A column label provider that can visualize tokens of a given {@link Witness}
 */
public class WitnessLabelProvider extends LabelProvider implements
IStyledLabelProvider, IColorProvider {

	/**
	 * Styler for tokens that are equivalent to the base text's entry at this
	 * position.
	 * 
	 * @see #getEquivalenceColor()
	 */
	public static Styler EQUAL_STYLER = new Styler() {

		@Override
		public void applyStyles(final TextStyle textStyle) {
			textStyle.foreground = getEquivalenceColor();
		}

	};

	private static Color getEquivalenceColor() {
		final String key = "equal-styler"; //$NON-NLS-1$
		if (!CollateXPlugin.getColorRegistry().hasValueFor(key))
			CollateXPlugin.getColorRegistry().put(key,
					new RGB(0xaa, 0xaa, 0xaa));
		return CollateXPlugin.getColorRegistry().get(key);
	}


	private final Witness witness;

	private final CollationResults collationResults;

	/**
	 * Creates a new label provider for the given witness.
	 * 
	 * @param collationResults
	 *            used to determine the current base text
	 * @param witness
	 *            the witness that we should label
	 */
	public WitnessLabelProvider(final CollationResults collationResults,
			final Witness witness) {
		this.witness = witness;
		this.collationResults = collationResults;
	}

	/**
	 * Returns <code>true</code> if the token we have to style is considered
	 * equivalent to the same row's entry for the currently focused (base)
	 * witness.
	 */
	private boolean isEquivalentToken(final Object element) {
		final Set<Token> tokens = WitnessLabelProvider.extractTokens(element,
				witness);
		final Witness focusedWitness = collationResults.getFocusedWitness();
		if (focusedWitness != null && !focusedWitness.equals(witness)) {
			final Set<Token> focusedTokens = WitnessLabelProvider
					.extractTokens(element, focusedWitness);
			if (focusedTokens != null
					&& tokens != null
					&& collationResults.getComparator().compare(
							focusedTokens.iterator().next(),
							tokens.iterator().next()) == 0)
				return true;
		}
		return false;
	}

	@Override
	public StyledString getStyledText(final Object element) {
		final Set<Token> tokens = WitnessLabelProvider.extractTokens(element,
				witness);

		// I believe we only ever get more than one token when we have collated
		// the same witness multiple times!?
		final String string = tokens == null ? "" : Joiner.on(" ").join( //$NON-NLS-1$ //$NON-NLS-2$
				Iterables.transform(tokens, new Function<Token, String>() {

					@Override
					public String apply(final Token input) {
						return ((SimpleToken) input).getContent();
					}

				}));

		if (isEquivalentToken(element))
			return new StyledString(string, StyledString.DECORATIONS_STYLER);
		else
			return new StyledString(string);

	}

	/**
	 * Extract the contents of the given table cell
	 * 
	 * @param rawRow
	 *            the row as passed into the Label- and ContentProvider methods.
	 *            Must be
	 *            <code>Map.Entry<Integer, Map<Witness, Set<Token>>></code>
	 * @param witness
	 *            the witness for which we would like to extract the tokens
	 * @return the set of tokens for the given cell.
	 */
	static Set<Token> extractTokens(final Object rawRow, final Witness witness) {
		@SuppressWarnings("unchecked")
		final Map.Entry<Integer, Map<Witness, Set<Token>>> row = (Entry<Integer, Map<Witness, Set<Token>>>) rawRow;
		final Set<Token> tokens = row.getValue().get(witness);
		return tokens;
	}

	@Override
	public Color getForeground(final Object element) {
		if (isEquivalentToken(element))
			return getEquivalenceColor();
		else
			return null;
	}

	@Override
	public Color getBackground(final Object element) {
		return null;
	}
}